package de.bsvrz.sys.usv.junit;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.usv.enums.BatterieZustand;
import de.bsvrz.sys.usv.enums.EingangsfrequenzZustand;
import de.bsvrz.sys.usv.enums.KommunikationsZustand;
import de.bsvrz.sys.usv.enums.USVZustand;
import de.bsvrz.sys.usv.enums.WechselSpannung;
import de.bsvrz.sys.usv.enums.ZustandNotstrom;
import de.bsvrz.sys.usv.enums.ZustandPhase;
import de.bsvrz.sys.usv.usv.DefaultUSV;
import de.bsvrz.sys.usv.wrapper.USVWrapper;

/**
 * Klasse erzeugt eine Test-USV. Sie erfüllt die volle Funktionalität des
 * Interface USVAllgemein. Über automatische Tests können die Werte die von
 * einer realen USV-Anlage geliefert werden simuliert werden, bzw. die Werte die
 * an eine reale USV-Anlage gesendet werden sollen, angezeigt werden. Alle
 * Testergebnise werden in einer Log-Datei mit Datum und Uhrzeit gespeichert.
 *
 * @author ObertM Geprüft 30.9.2008 Hilpp
 *
 */

public class SimuUSVTest extends DefaultUSV implements Runnable, ClientReceiverInterface, ClientSenderInterface {

	private USVZustand m_UsvZustand = USVZustand.na;
	private int m_BatterieZustand = BatterieZustand.na.getValue();
	private KommunikationsZustand m_KommunikationsZustand = KommunikationsZustand.na;
	private EingangsfrequenzZustand m_EingangsFrequenzZustand = EingangsfrequenzZustand.na;
	private WechselSpannung m_WechselSpannung = WechselSpannung.na;
	private ZustandPhase m_ZustandPhase1 = ZustandPhase.na;
	private ZustandPhase m_ZustandPhase2 = ZustandPhase.na;
	private ZustandPhase m_ZustandPhase3 = ZustandPhase.na;
	private ZustandNotstrom m_ZustandNotstrom = ZustandNotstrom.na;

	private final ClientDavInterface m_connection;
	private final String m_pidUsv;

	private USVWrapper m_instance;
	private final SimuUSVTest m_instanceSimuUsvTest;
	private String m_attributgruppe;
	private String m_wert;

	private String m_IUsvZustand;
	private String m_IBatterieZustand;
	private String m_IKommunikationsZustand;
	private String m_IEingangsFrequenzZustand;
	private String m_IWechselSpannung;
	private String m_IZustandPhase1;
	private String m_IZustandPhase2;
	private String m_IZustandPhase3;
	private String m_IZustandNotstrom;
	private String m_IKritischerZustand;

	private final Debug _debug;

	/**
	 * Konstruktor der Klasse
	 *
	 */
	public SimuUSVTest(final ClientDavInterface connection, final String pid_USV, final String test) {
		m_connection = connection;
		m_pidUsv = pid_USV;
		m_instanceSimuUsvTest = this;

		_debug = Debug.getLogger();

		final String v[] = test.split(":");

		if (v.length == 2) {
			m_attributgruppe = v[0];
			m_wert = v[1];
			System.out.println("SimuUSVTest-> Attributgruppe: " + m_attributgruppe + " Wert: " + m_wert);
		} else {
			System.out.println("SimuUSVTest-> Falsche Parameteranzahl ");
			System.exit(0);
		}

		final Thread thread = new Thread(this);
		thread.setName(this.getClass().getName());

		thread.start();

	}

	/**
	 * Methode meldet sich beim Datenverteiler an.
	 *
	 * @param connection
	 *            die Datenverteilerverbindung
	 * @param pid_USV
	 *            die PID des USV-Objekts
	 * @param atgusv
	 *            die PID der USV-Attributgruppe
	 */
	public void davAnmelden(final ClientDavInterface connection, final String pid_USV, final String atgusv) {
		// ---------------------------------------------------------------------------
		// Anmeldung auf Attributgruppe atg.usvBefehl mit dem Aspekt asp.vorgabe
		// ---------------------------------------------------------------------------

		System.out.println("SimuUSVTest angemeldet Pid: " + pid_USV);

		final AttributeGroup atg = connection.getDataModel().getAttributeGroup(atgusv);
		final Aspect asp = connection.getDataModel().getAspect("asp.zustand");

		final DataDescription dd = new DataDescription(atg, asp);

		final SystemObject usv = connection.getDataModel().getObject(m_pidUsv);
		connection.subscribeReceiver(this, usv, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
	}

	/**
	 * Methode zum Anmelden und Senden von Notstrom und USV-Betrieb
	 *
	 * @param connection
	 *            die Datenverteilerverbindung
	 * @param pid_USV
	 *            die PID des USV-Objekts
	 * @param atgusv
	 *            die PID der USV-Attributgruppe
	 */
	public DataDescription davSendeAnmelden(final ClientDavInterface connection, final String pid_USV,
			final String atgusv) {
		final ClientDavInterface conSenden = connection;

		final AttributeGroup atg = conSenden.getDataModel().getAttributeGroup(atgusv);
		final Aspect asp = conSenden.getDataModel().getAspect("asp.vorgabe");

		final DataDescription dd = new DataDescription(atg, asp);
		final SystemObject usvSend = conSenden.getDataModel().getObject(pid_USV);

		try {
			conSenden.subscribeSender(this, usvSend, dd, SenderRole.sender());
		} catch (final OneSubscriptionPerSendData e1) {
			e1.printStackTrace();
		}

		// Senden der daten

		final Data d1 = conSenden.createData(atg);

		if (m_wert.equalsIgnoreCase("Ja")) {
			d1.getScaledValue("Notstrom").setText("Ja");
			d1.getScaledValue("UsvBetrieb").setText("Nein");
		} else {
			d1.getScaledValue("Notstrom").setText("Nein");
			d1.getScaledValue("UsvBetrieb").setText("Ja");
		}

		final ResultData rsd = new ResultData(usvSend, dd, new Date().getTime(), d1);

		try {
			conSenden.sendData(rsd);
		} catch (final DataNotSubscribedException e1) {
			e1.printStackTrace();
		} catch (final SendSubscriptionNotConfirmed e1) {
			e1.printStackTrace();
		}

		return dd;
	}

	/**
	 * Methode wird alle 10 Sekunden gestartet
	 */
	@Override
	public void run() {

		if (m_attributgruppe.equalsIgnoreCase("usvZustand")) {
			davAnmelden(m_connection, m_pidUsv, "atg.usvZustand");
		}

		if (m_attributgruppe.equalsIgnoreCase("usvZustandKritisch")) {
			davAnmelden(m_connection, m_pidUsv, "atg.usvZustandKritisch");
		}

		if (m_attributgruppe.equalsIgnoreCase("usvBefehl")) {
			final DataDescription dd = davSendeAnmelden(m_connection, m_pidUsv, "atg.usvBefehl");
		}

		while (true) {

			if (m_attributgruppe.equalsIgnoreCase("usvZustandKritisch")) {
				m_UsvZustand = USVZustand.Batterie;
				m_instanceSimuUsvTest.getInstance().setTrigger();

				m_BatterieZustand = Integer.parseInt(m_wert);
				m_instanceSimuUsvTest.getInstance().setTrigger();

				sleep(1);
				pruefe("usvZustandKritisch", "Bei " + m_BatterieZustand + " %", null, null);

			}

			if (m_attributgruppe.equalsIgnoreCase("usvZustand")) {
				// ------------------------------------------------------------------------------
				// Prüfe Batteriezustand
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("batteriezustand") || m_wert.equalsIgnoreCase("all")) {
					int zaehler = 0;
					for (int i = 0; i <= 100; i++) {
						m_BatterieZustand = zaehler;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", "" + m_BatterieZustand + " %", " Batteriezustand: ", m_IBatterieZustand);
						zaehler++;
					}

					m_BatterieZustand = BatterieZustand.na.getValue();
					m_instanceSimuUsvTest.getInstance().setTrigger();
					sleep(1);
					pruefe("usvZustand", "nicht ermittelbar", " Batteriezustand: ", m_IBatterieZustand);
				}

				// ------------------------------------------------------------------------------
				// Prüfe Usvzustand
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("usvzustand") || m_wert.equalsIgnoreCase("all")) {
					for (final USVZustand z : USVZustand.values()) {
						m_UsvZustand = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Usvzustand: ", m_IUsvZustand);
					}
				}

				// ------------------------------------------------------------------------------
				// Prüfe Kommunikationszustand
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("kommunikationszustand") || m_wert.equalsIgnoreCase("all")) {
					for (final KommunikationsZustand z : KommunikationsZustand.values()) {
						m_KommunikationsZustand = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Kommunikationszustand: ", m_IKommunikationsZustand);
					}
				}
				// ------------------------------------------------------------------------------
				// Prüfe EingangsfrequenzZustand
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("eingangsfrequenzZustand") || m_wert.equalsIgnoreCase("all")) {
					for (final EingangsfrequenzZustand z : EingangsfrequenzZustand.values()) {
						m_EingangsFrequenzZustand = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " EingangsfrequenzZustand: ", m_IEingangsFrequenzZustand);
					}
				}

				// ------------------------------------------------------------------------------
				// Prüfe EingangsfrequenzZustand
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("wechselspannung") || m_wert.equalsIgnoreCase("all")) {
					for (final WechselSpannung z : WechselSpannung.values()) {
						m_WechselSpannung = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Wechselspannung: ", m_IWechselSpannung);
					}
				}
				// ------------------------------------------------------------------------------
				// Prüfe Zustand Phase 1
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("zustandphase1") || m_wert.equalsIgnoreCase("all")) {
					for (final ZustandPhase z : ZustandPhase.values()) {
						m_ZustandPhase1 = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Zustand Phase 1: ", m_IZustandPhase1);
					}
				}
				// ------------------------------------------------------------------------------
				// Prüfe Zustand Phase 2
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("zustandphase2") || m_wert.equalsIgnoreCase("all")) {
					for (final ZustandPhase z : ZustandPhase.values()) {
						m_ZustandPhase2 = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Zustand Phase 2: ", m_IZustandPhase2);
					}
				}
				// ------------------------------------------------------------------------------
				// Prüfe Zustand Phase 3
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("zustandphase3") || m_wert.equalsIgnoreCase("all")) {
					for (final ZustandPhase z : ZustandPhase.values()) {
						m_ZustandPhase3 = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Zustand Phase 3: ", m_IZustandPhase3);
					}
				}

				// ------------------------------------------------------------------------------
				// Prüfe Zustand Notstrom
				// ------------------------------------------------------------------------------

				if (m_wert.equalsIgnoreCase("zustandnotstrom") || m_wert.equalsIgnoreCase("all")) {
					for (final ZustandNotstrom z : ZustandNotstrom.values()) {
						m_ZustandNotstrom = z;
						m_instanceSimuUsvTest.getInstance().setTrigger();
						sleep(1);
						pruefe("usvZustand", z.getText(), " Zustand Notstrom: ", m_IZustandNotstrom);
					}
				}

			} // if Zustandsprüfung

			_debug.info("SimuUSVTest ->  Testlauf beendet ");

			while (true) {
				// Testlauf beendet
				try {
					Thread.sleep(5000); // Warte bis USV - sendebereit
				} catch (final InterruptedException e1) {
				}
			}

		} // Ende While(true)
	}

	/**
	 * Methode vergleicht Befehl mit Rückmeldung
	 *
	 * @param atg
	 * @param zustand
	 * @param text
	 */
	public void pruefe(final String atg, final String zustand, final String text, final String rueckmeldung) {

		if (atg.equalsIgnoreCase("usvZustand")) {
			if (zustand.equalsIgnoreCase(rueckmeldung)) {
				_debug.info(
						"SimuUSVTest -> Attributgruppe UsvZustand: " + text + "" + zustand + "  -> Ergebnis richtig ");
			} else {
				_debug.info(
						"SimuUSVTest -> Attributgruppe UsvZustand: " + text + "" + zustand + "  -> Ergebnis falsch ");
			}
		}

		if (atg.equalsIgnoreCase("usvZustandKritisch")) {
			_debug.info("SimuUSVTest -> Attributgruppe Kritischer Zustand: " + zustand + "  " + m_IKritischerZustand);
		}
	}

	/**
	 * Updatemethode
	 */
	@Override
	public void update(final ResultData[] arg0) {
		for (final ResultData data : arg0) {

			// ---------------------------------------------------------------------------------
			// Auswerten der Attributgruppe atg.usvZustand
			// ---------------------------------------------------------------------------------

			if (data.getDataDescription().getAttributeGroup().getPid().equals("atg.usvZustand")) {
				if (data.hasData()) {
					m_IBatterieZustand = data.getData().getScaledValue("BatterieZustand").getText();
					m_IUsvZustand = data.getData().getScaledValue("UsvZustand").getText();
					m_IKommunikationsZustand = data.getData().getScaledValue("KommunikationsZustand").getText();
					m_IEingangsFrequenzZustand = data.getData().getScaledValue("EingangsFrequenzZustand").getText();
					m_IWechselSpannung = data.getData().getScaledValue("WechselSpannung").getText();
					m_IZustandPhase1 = data.getData().getScaledValue("ZustandPhase1").getText();
					m_IZustandPhase2 = data.getData().getScaledValue("ZustandPhase2").getText();
					m_IZustandPhase3 = data.getData().getScaledValue("ZustandPhase3").getText();
					m_IZustandNotstrom = data.getData().getScaledValue("ZustandNotStrom").getText();
				}
			}

			if (data.getDataDescription().getAttributeGroup().getPid().equals("atg.usvZustandKritisch")) {
				if (data.hasData()) {
					m_IKritischerZustand = data.getData().getScaledValue("KritischerZustand").getText();
				}
			}

		}
	}

	/**
	 * Sleep für Sekundenintervalle
	 *
	 * @param intervall
	 */
	public void sleep(final int intervall) {
		try {
			Thread.sleep(intervall * 1000);
		} catch (final InterruptedException e) {
			// TODO Automatisch erstellter Catch-Block
			e.printStackTrace();
		}
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_UsvZustand
	 */
	@Override
	public USVZustand getUsvZustand() {
		return m_UsvZustand;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_BatterieZustand
	 */
	@Override
	public int getBatterieZustand() {
		return m_BatterieZustand;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_KommunikationsZustand
	 */
	@Override
	public KommunikationsZustand getKommunikationsZustand() {
		return m_KommunikationsZustand;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_EingangsFrequenzZustand
	 */
	@Override
	public EingangsfrequenzZustand getEingangsFrequenzZustand() {
		return m_EingangsFrequenzZustand;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_WechselSpannung
	 */
	@Override
	public WechselSpannung getWechselSpannung() {
		return m_WechselSpannung;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_ZustandPhase1
	 */
	@Override
	public ZustandPhase getZustandPhase1() {
		return m_ZustandPhase1;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_ZustandPhase2
	 */
	@Override
	public ZustandPhase getZustandPhase2() {
		return m_ZustandPhase2;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_ZustandPhase2
	 */
	@Override
	public ZustandPhase getZustandPhase3() {
		return m_ZustandPhase3;
	}

	/**
	 * Methode zum Aulesen der Membervariablen
	 *
	 * @return m_ZustandPhase3
	 */
	@Override
	public ZustandNotstrom getZustandNotstrom() {
		return m_ZustandNotstrom;
	}

	/**
	 * Methode zum Setzen der Membervariablen m_BefehlNotstrom
	 *
	 * @return 0  Funktion erfolgreich durchgeführt 1  Funktion nicht
	 *         erfolgreich durchgeführt -1  Funktionalität nicht implementiert.
	 */
	@Override
	public int setNotstrom(final boolean befehl) {
		if (befehl == true) {
			_debug.info("SimuUSVTest -> Attributgruppe UsvBefehl:   Schalte Notstrom ein ");
		} else {
			_debug.info("SimuUSVTest -> Attributgruppe UsvBefehl:   Schalte Notstrom aus ");
		}

		return 0;
	}

	/**
	 * Methode zum Setzen der Membervariablen m_BefehlNotstrom
	 *
	 * @return 0  Funktion erfolgreich durchgeführt 1  Funktion nicht
	 *         erfolgreich durchgeführt -1  Funktionalität nicht implementiert.
	 */
	@Override
	public int setUsvBetrieb(final boolean befehl) {
		if (befehl == true) {
			_debug.info("SimuUSVTest -> Attributgruppe UsvBefehl:   Schalte Usv-Betrieb ein");
		} else {
			_debug.info("SimuUSVTest -> Attributgruppe UsvBefehl:   Schalte Usv-Betrieb aus");
		}

		return 0;
	}

	/**
	 * Methode zum Setzen der Membervariablen m_instance
	 */
	@Override
	public void setInstance(final USVWrapper instance) {
		m_instance = instance;
	}

	/**
	 * Methode zum Lesen der Membervariablen m_instance
	 *
	 * @return Instanz der Hauptklasse USVWrapper
	 */

	public USVWrapper getInstance() {
		return m_instance;
	}

	/**
	 * Methode zur Ausgabe des aktuellen Datums und Uhrzeit
	 *
	 * @return String mit Datum und Uhrzeit
	 */
	public static String time() {
		final Calendar cal = Calendar.getInstance();
		final DateFormat formatter = new SimpleDateFormat();

		return (formatter.format(cal.getTime()) + " ");
	}

	@Override
	public void dataRequest(final SystemObject arg0, final DataDescription arg1, final byte arg2) {
		// TODO Automatisch erstellter Methoden-Stub

	}

	@Override
	public boolean isRequestSupported(final SystemObject arg0, final DataDescription arg1) {
		// TODO Automatisch erstellter Methoden-Stub
		return false;
	}
}
