/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.sys.usv.wrapper;

import java.lang.Thread.UncaughtExceptionHandler;
import java.lang.reflect.Constructor;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageCauser;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.sys.funclib.operatingMessage.MessageState;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;

/**
 * Das Modul USV-Wrapper realisiert folgende Funktionalitäten: - Verbinden und
 * Austausch der Daten mit dem Datenverteiler. Die Klasse implementiert das
 * Interface StandardApplication und wird mit dem StandardApplikationRunner
 * instanziert. Dadurch wird die Verbindung zum DaV nach dem Start automatisch
 * aufgebaut. - Zyklisches Einlesen und Auswerten der Zustände der USV. Die
 * Zustände der USV müssen zyklisch eingelesen und auf Änderung hin überprüft
 * werden. Das Einlesen der Daten erfolgt über eine abstrakte Schnittstelle die
 * die Daten vom Modul USV übernimmt. Bei Änderungen innerhalb der Zustände
 * müssen folgende Aktivitäten durchgeführt werden: - Erzeugen einer
 * Attributgruppe atg.usvZustand mit den Zustände der USV und Versenden dieser
 * Attributgruppe an den Datenverteiler - Erzeugen von Betriebsmeldungen und
 * Versenden an den Datenverteiler über den MessageSender - Prüfen ob sich die
 * USV in einem kritischen Zustand befindet (z. B. Batterie schwach). Wenn ja,
 * dann muss ein Beenden der Applikationen und ein gezieltes Herunterfahren des
 * Rechners eingeleitet werden (das Beenden bzw. das Herunterfahren sind
 * Aufgaben der SWE Start/Stop). - Senden empfangener Befehle an die USV Über
 * die abstrakte Schnittstelle zum Modul USV werden mögliche Befehle für die USV
 * an die USV abgesetzt.
 *
 * @author M.Obert Geprüft 30.9.2008 Hilpp
 * @author Christian Hösel, BitCtrl Systems GmbH
 */
public class USVWrapper implements StandardApplication, ClientReceiverInterface, USVTrigger {

	private static final String ASP_VORGABE = "asp.vorgabe";

	private static final String ATG_USV_BEFEHL = "atg.usvBefehl";

	/**
	 * Der Dav-Logger.
	 */
	private final Debug log = Debug.getLogger();

	/**
	 * Verbindung zum Datenverteiler
	 */
	private static ClientDavInterface m_connection;

	/**
	 * Objekt SenderThread
	 */
	public SenderThread leseUSV;

	/**
	 * Name der von DefaultUSV abgeleiteten Klasse
	 */
	private String usvKlasse;

	/**
	 * Klasse der von DefaultUSV abgeleiteten Klasse
	 */
	private USVAllgemein usvB;

	/**
	 * Protokoll und Adresse des SNMP-Servers.
	 */
	private static String snmpServer;

	/**
	 * Pid der verwendeten USV-Anlage
	 */
	private static String m_pidUsv;

	private String m_befehlNotstrom = "";
	private String m_befehlUsv = "";

	private String m_oldBefehlNotstrom = "";
	private String m_oldBefehlUsv = "";
	private int m_zyklusZeit;

	private String m_test;

	/**
	 * Schwellwerte zum Auswerten bzw. Auslösen der Attributgruppe
	 * USV-kritischer Zustand
	 */
	private String schwellwertParameter;

	/**
	 * Timeout in Millisekunden, der bei der Kommunikation mit der USV verwandt
	 * wird.
	 */
	private long usvTimeout;

	/*
	 * (non-Javadoc)
	 *
	 * @see sys.funclib.application.StandardApplication#initialize(stauma.dav.
	 * clientside .ClientDavInterface)
	 */
	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		USVWrapper.m_connection = connection;

		final SystemObject usv = connection.getDataModel().getObject(USVWrapper.m_pidUsv);
		if (usv == null) {
			log.error("Das übergebene USV-Objekt '" + USVWrapper.m_pidUsv + "' existiert nicht!");
			System.exit(-1);
		}

		log.info("Gefundenes USV-Dav-Objekt: " + usv);

		// Klasse bestimmen aus Übergabeparameter
		final Class<?> clazz = Class.forName(usvKlasse);

		/**
		 * Wir akzetieren nur 'Plugins', die das richtige Interface
		 * implementieren!!
		 */
		if (!USVAllgemein.class.isAssignableFrom(clazz)) {
			throw new IllegalArgumentException("Der Parameter '-usvKlasse' beschreibt keine Klasse die das Interface "
					+ USVAllgemein.class.getCanonicalName() + " implementiert.");
		}

		if (m_test == null) {
			// Konstruktor der Klasse bestimmen
			final Constructor<?> con = clazz.getConstructor();

			// Neue Instanz der Klasse mit diesem Konstruktor bilden
			usvB = (USVAllgemein) con.newInstance();
		} else {
			usvB = (USVAllgemein) clazz.getConstructor(ClientDavInterface.class, String.class, String.class)
					.newInstance(new Object[] { USVWrapper.m_connection, USVWrapper.m_pidUsv, m_test });
		}

		// Der Klasse die Instanz von USVWrapper übergeben, damit diese die
		// Methode setTrigger aufrufen kann.
		usvB.setInstance(this);

		leseUSV = new SenderThread(usvB, USVWrapper.m_pidUsv, schwellwertParameter, m_zyklusZeit);

		leseUSV.setUncaughtExceptionHandler(new UncaughtExceptionHandler() {
			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				Debug.getLogger().error(
						"Der Thread zur zyklischen Aktualisierung der USV - Daten und damit die gesamte Anwendung wurden unerwartet beendet.",
						e);
				System.exit(-1);
			}
		});
		leseUSV.start();

		// ---------------------------------------------------------------------------
		// Anmeldung auf Attributgruppe atg.usvBefehl mit dem Aspekt asp.vorgabe
		// ---------------------------------------------------------------------------
		final AttributeGroup atg_usvBefehl = connection.getDataModel().getAttributeGroup(USVWrapper.ATG_USV_BEFEHL);
		final Aspect asp_vorgabe = connection.getDataModel().getAspect(USVWrapper.ASP_VORGABE);

		connection.subscribeReceiver(this, usv, new DataDescription(atg_usvBefehl, asp_vorgabe),
				ReceiveOptions.normal(), ReceiverRole.drain());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * sys.funclib.application.StandardApplication#parseArguments(sys.funclib
	 * .ArgumentList)
	 */
	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		usvKlasse = argumentList.fetchArgument("-usvKlasse").asNonEmptyString();
		USVWrapper.m_pidUsv = argumentList.fetchArgument("-usv").asNonEmptyString();
		schwellwertParameter = argumentList.fetchArgument("-schwelleHerunterfahren").asNonEmptyString();
		final String zz = argumentList.fetchArgument("-zyklusZeit").asNonEmptyString();

		USVWrapper.snmpServer = argumentList.fetchArgument("-snmp").asNonEmptyString();

		usvTimeout = argumentList.fetchArgument("-timeout=" + USVAllgemein.DEFAULT_TIMEOUT).longValue();

		m_zyklusZeit = Integer.parseInt(zz);

		if (argumentList.hasArgument("-test")) {
			m_test = argumentList.fetchArgument("-test").asNonEmptyString();
		} else {
			m_test = null;
		}
		argumentList.fetchUnusedArguments();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.bsvrz.sys.usv.wrapper.USVTrigger#setTrigger()
	 */
	@Override
	public synchronized void setTrigger() {
		leseUSV.setTrigger();
	}

	/**
	 * Main Methode der Applikation. Startet den StandardApplicationRunner.
	 *
	 * @param arguments
	 *            Aufrufparameter
	 */
	public static void main(final String[] arguments) {

		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {

			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				Logger.getLogger(getClass().getName()).log(Level.INFO, "*** Application wird beendet ***", e); //$NON-NLS-1$
			}

		});
		StandardApplicationRunner.run(new USVWrapper(), arguments);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.
	 * clientside .ResultData[])
	 */
	@Override
	public void update(final ResultData[] arg0) {
		for (final ResultData data : arg0) {
			// ---------------------------------------------------------------------------------
			// Auswerten der Attributgruppe atg.usvBefehl
			// ---------------------------------------------------------------------------------
			if (data.getDataDescription().getAttributeGroup().getPid().equals(USVWrapper.ATG_USV_BEFEHL)) {
				if (data.hasData()) {
					m_befehlNotstrom = data.getData().getScaledValue("Notstrom").getText();
					m_befehlUsv = data.getData().getScaledValue("UsvBetrieb").getText();

					if (!m_befehlNotstrom.equals(m_oldBefehlNotstrom)) {
						usvB.setNotstrom(m_befehlNotstrom.equals("Ja"));
						protokolliereBefehlUsv("Befehl Notstrom", m_befehlNotstrom, m_oldBefehlNotstrom);
						m_oldBefehlNotstrom = m_befehlNotstrom;
					}

					if (!m_befehlUsv.equals(m_oldBefehlUsv)) {
						usvB.setUsvBetrieb(m_befehlUsv.equals("Ja"));
						protokolliereBefehlUsv("Befehl UsvBetrieb", m_befehlUsv, m_oldBefehlUsv);
						m_oldBefehlUsv = m_befehlUsv;
					}
				}
			}
		}
	}

	/**
	 * Methode zum Lesen der aktuellen Datenverteilerverbindung
	 *
	 * @return Datenverteilerverbindung
	 */
	public static ClientDavInterface getConnection() {
		return USVWrapper.m_connection;

	}

	/**
	 * Methode zum Absetzen einer Betriebsmeldung an den Datenverteiler
	 *
	 * @param objekt
	 *            Name des sich änderenden Objekts
	 * @param zustandNeu
	 *            Neuer Zustand des Objekts
	 * @param zustandAlt
	 *            Alter Zustand des Objekts
	 */
	private void protokolliereBefehlUsv(final String objekt, final String zustandNeu, final String zustandAlt) {
		final String text = objekt + " Wechsel von \"" + zustandAlt + "\" nach \"" + zustandNeu + "\"";

		log.info("Versende neu Betriebsmeldung mit folgendem Inhalt: " + text);

		final MessageType typ = MessageType.SYSTEM_DOMAIN;
		final String messageTypeAddOn = "Wechsel";
		final MessageGrade grade = MessageGrade.INFORMATION;
		final String message = text;

		sendMessage(typ, messageTypeAddOn, grade, message);
	}

	/**
	 * Methode erzeugt eine Betriebsmeldung und sendet diese an den
	 * Datenverteiler. Eingestellt werden können MeldungsTyp, MeldungsTypZusatz
	 * und die Meldungsklasse.
	 *
	 * @param type
	 *            der MeldungsTyp
	 * @param messageTypeAddOn
	 *            der MeldungsTypZusatz
	 * @param grade
	 *            die MeldungsKlasse
	 * @param message
	 *            Text der Meldung
	 */
	public void sendMessage(final MessageType type, final String messageTypeAddOn, final MessageGrade grade,
			final String message) {
		final MessageSender msg = MessageSender.getInstance();

		if (msg == null) {
			log.warning("MessageSender = null --> es können keine Betriebsmeldungen versandt werden,");
			return;
		}

		msg.setApplicationLabel(USVWrapper.m_pidUsv);

		final String id = "" + USVWrapper.m_connection.getLocalApplicationObject().getId();

		final SystemObject referenceObject = null;
		final MessageState state = MessageState.CHANGE_MESSAGE;
		final MessageCauser causer = new MessageCauser(null, "", "");

		/*
		 * @param id ID der Meldung. Dieses Attribut kann von der Applikation
		 * gesetzt werden, um einen Bezug zu einer vorherigen Meldung
		 * herzustellen. @param type der MeldungsTyp @param messageTypeAddOn der
		 * MeldungsTypZusatz @param grade die MeldungsKlasse @param
		 * referenceObject Referenz auf ein beliebiges Konfigurationsobjekt, auf
		 * das sich die Meldung bezieht. @param state Gibt den Zustand einer
		 * Meldung an. @param causer Urlasserinformation (Referenz auf den
		 * Benutzer, der die Meldung erzeugt hat, Angabe einer Ursache für die
		 * Meldung und der Veranlasser für die Meldung) @param message Text der
		 * Meldung
		 */

		msg.sendMessage(id, type, messageTypeAddOn, grade, referenceObject, state, causer, message);
	}

	/**
	 * @return liefert die Variable pidUsv zurück
	 */
	public static String getPidUsv() {
		return USVWrapper.m_pidUsv;
	}

	/**
	 * Erfragt Protokoll und Adresse des SNMP-Servers.
	 *
	 * @return Protokoll und Adresse des SNMP-Servers.
	 */
	public static String getSnmpServer() {
		return USVWrapper.snmpServer;
	}

	/**
	 * Liefert den Timeout in Millisekunden, der bei der Kommunikation mit der
	 * USV verwandt wird.
	 *
	 * @return Timout in Millisekunden.
	 */
	public final long getUsvTimeout() {
		return usvTimeout;
	}

}
